#!/usr/bin/env python

#
# welcome.py
# This file is part of Welcome-Centre
#
# Copyright (C) 2006 - Xubuntu Welcome-Centre Team
#
# Welcome-Centre is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Welcome-Centre is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Welcome-Centre; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, 
# Boston, MA  02110-1301  USA
#  

import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade
import gtkhtml2
import stat
import os
import os.path
import gettext
import gobject

# Localisations
gettext.bindtextdomain('welcome-centre')
gettext.textdomain('welcome-centre')
_ = gettext.gettext

launcher_file = os.path.expanduser('~/.config/autostart/welcome.desktop')
default_lang = 'lt'

launcher_text = """
[Desktop Entry]
Encoding=UTF-8
Name=Welcome Center
Icon=help
Type=Application
Exec=welcome
"""

class WelcomeWindow:
    def __init__(self):
        """Create the main window."""

        self.xml = gtk.glade.XML('welcome.glade')
        self.xml.signal_autoconnect(self)
        os.chdir("./html/server/") #+default_lang

        # Set the checkbox
        check_show = self.xml.get_widget('check_show')
        check_show.set_active(self.get_show_on_login())

        # Prepare the gtkhtml2 widgets
        self.document = gtkhtml2.Document()
        self.view = gtkhtml2.View()
        self.view.connect('on_url', self.on_url)
        scrolled_window = self.xml.get_widget('html_display')
        scrolled_window.set_hadjustment(self.view.get_hadjustment())
        scrolled_window.set_vadjustment(self.view.get_vadjustment())
        self.view.set_document(self.document)
        scrolled_window.add(self.view)
        scrolled_window.show_all()
        
        self.statusbar = self.xml.get_widget('statusbar1')
        self.error_context = self.statusbar.get_context_id("error")
        self.status_context = self.statusbar.get_context_id("status")
        self.link_context = self.statusbar.get_context_id("link")

        toolbar = self.xml.get_widget('toolbar1')
        
        if gtk.gdk.screen_get_default().get_height() > 580:
            # Resize the icons
            for toolitem in toolbar:
                if toolitem.get_name().startswith('button_'):
                    icon = gtk.image_new_from_icon_name(
                        toolitem.get_icon_name(), 48)
                    icon.set_pixel_size(48)
                    toolitem.set_icon_widget(icon)

        # Gray out inactive buttons and get the starting page
        self.starting_page = "apie-baltix_server.html"
        for toolitem in toolbar:
            name = toolitem.get_name()
            if name.startswith('button_'):
                filename = self.get_file_name(name)
                if os.path.exists(filename):
                    if self.starting_page == "":
                        self.starting_page = filename
                else:
                    toolitem.set_sensitive(False)
        toolbar.show_all()
            
        # Connect the events
        self.document.connect('link_clicked', self.on_document_link_clicked)
        self.document.connect('request_url', self.on_document_request_url)
        
        # Read the text of starting page.
        self.current_file = ""
        self.read_file(self.starting_page)
        
        

    def set_text(self, text):
        """Set the html text displayed in the left panel."""
        
        self.document.clear()
        self.document.open_stream('text/html')
        self.document.write_stream(text)
        self.document.close_stream()
        self.view.get_hadjustment().set_value(0)
        self.view.get_vadjustment().set_value(0)

    def show_error(self, error):
        self.statusbar.push(self.error_context, error)

    def show_status(self, status):
        self.statusbar.push(self.status_context, status)
    
    def clear_error(self):
        """Pop the status bar message."""
        self.statusbar.pop(self.error_context)
        self.statusbar.pop(self.status_context)

    def read_file(self, file_name):
        """Read a file and display it in the left panel."""
               
        self.clear_error()
        if file_name == self.current_file:
            return False    
        
        try:
            f = file(file_name)
            text = f.read()
            self.set_text(text)
            self.current_file = file_name
        except:
            self.show_error(_('Error reading file  "%s".') % file_name)

    def get_show_on_login(self):
        """Check the existence and permissions of the launcher file."""

        try:
            s = os.stat(launcher_file)
            if stat.S_IMODE(s.st_mode):
                return True
        except OSError:
            pass
        return False

    def set_show_on_login(self, show):
        if show:
            # Create proper .desktop file with correct permissions
            try:
                os.chmod(launcher_file, stat.S_IRWXU)
            except OSError:
                try:
		    if not os.path.isdir (os.path.expanduser('~/.config/autostart/')):
	        	os.makedirs(os.path.expanduser('~/.config/autostart/'))
                    f = file(launcher_file, 'w')
                    f.write(launcher_text)
                    f.close()
                    os.chmod(launcher_file, stat.S_IRWXU)
                except OSError:
                    self.show_error('Warning: Unable to create launcher file "%s".' % launcher_file)
                    
        else:
            # Remove the permissions from the .desktop file
            try:
                os.chmod(launcher_file, 0)
            except OSError:
                pass

    def get_file_name(self, widget_name):
        """Compute the file name from the clicked button name."""

        if widget_name.startswith('button_'):
            name = "%s.html" % widget_name[len('button_'):] ## hack LANG by dz0
        else:
            name = self.starting_page
        return _(name)
        

# ------------ Event handlers -----
    def on_url(self, view, link):
        if link:
            if link.startswith('run://'):
                command = link[len('run://'):]
                link = _('Execute "%s"') % command
            if link.startswith('run-terminal://'):
                command = link[len('run-terminal://'):]
                link = _('Execute "%s" in terminal') % command
            elif link.startswith('http://') or link.startswith('ftp://') or link.startswith("https://")  or link.startswith("file://"):
                link = _('Open in browser %s') % link
            self.statusbar.push(self.link_context, link)
        else:
            self.statusbar.pop(self.link_context)
    

    def on_main_window_destroy(self, window):
        gtk.main_quit()
    
    on_button_close_clicked = on_main_window_destroy

    def on_button_clicked(self, button):
        """Assign a file depending on the widget name."""
        self.clear_error()
        file_name = self.get_file_name(button.get_name())
        self.read_file(file_name)

    def on_check_show_toggled(self, checkbox):
        self.clear_error()
        self.set_show_on_login(checkbox.get_active())

    def on_document_request_url(self, document, link, stream):
        try:
            f = file(link)
            data = f.read()
            stream.write(data)
        except:
            self.show_error(_("""Warning: Can't read file "%s".""") % link)
        stream.close()

    def on_document_link_clicked(self, document, link):
        """Handle links. Directly open local files.
           Run Firefox for http:// and ftp://, launch apps for run:"""

        self.clear_error()
	print link;
        if link[0]=="#": 
            self.view.jump_to_anchor(link[1:])
	    print link;
	elif link.startswith('http://') or link.startswith('ftp://') or link.startswith("https://") or link.startswith("file://"):
            # start default web browser
            self.show_status(_('Opening in browser "%s"...') % link)
            os.spawnl(os.P_NOWAIT, '/usr/bin/x-www-browser', '', link)
        elif link.startswith('run://'):
            # launch program
            command = link[len('run://'):]
            self.show_status(_('Executing "%s"...') % command)
            #os.spawnl(os.P_NOWAIT, '/bin/bash', '/bin/bash', command)
            os.spawnlp (os.P_NOWAIT, command, command)
        elif link.startswith('run-terminal://'):
            # launch program
            command = link[len('run-terminal://'):]
            self.show_status(_('Executing "%s"  in terminal ...') % command)
            os.spawnlp(os.P_NOWAIT, 'x-terminal-emulator', 'x-terminal-emulator', '-e', command)
        else:
            if link[0]=='#': return; ## hack anchor problem
            if '#' in link:
            	link, anchor = link.split('#')[:2]
            	self.read_file(link)
            	self.view.jump_to_anchor(anchor)
            else: self.read_file(link)
            
             
win = WelcomeWindow()
gtk.main()
